// WatchDog.cpp

#include "watchdog.h"

#define PAGE_SIZE 				0x1000 	// 1 page:4KB
#define PAGE_ALIGN(Vaddr) 		((PVOID)((ULONG)(Vaddr) & ~(PAGE_SIZE - 1)))
#define ROUND_TO_PAGES(Size)  	(((ULONG)(Size) + PAGE_SIZE - 1) & ~(PAGE_SIZE - 1))

extern "C"
{
   __declspec(dllimport) BOOL   VirtualCopy(LPVOID lpvDest, LPVOID lpvSrc, DWORD cbSize, DWORD fdwProtect);
}

// forward declarations
PVOID MapPhysicalAddr(PVOID dwPhysicalAddress, DWORD dwSize);


WatchDog::WatchDog(ULONG interval)
{
	LONG oscr;
	ULONG osmr3;
	ULONG oier;
	
	m_pOST = (OST *)MapPhysicalAddr((void *)XSCALE_OST_PHYSICAL_BASE, 0x1000);

	DEBUGMSG(TRUE, (_T("+WatchDog::WatchDog(0x%08x)\r\n"), interval));
	// save the interval in processor ticks
	m_interval = (ULONG)(interval * INPUT_CLOCK);

	// Read Current OS Timer Counter
	oscr = (LONG)READ_REGISTER_ULONG(&m_pOST->OSCR);
	DEBUGMSG(TRUE, (_T("\tOSCR = 0x%x\r\n"), oscr));

	// increment by timeout
	osmr3 = oscr + m_interval;
	DEBUGMSG(TRUE, (_T("\tSetting OSMR3 = 0x%x\r\n"), osmr3));

	// write to the watchdog register (osmr3)
	WRITE_REGISTER_ULONG(&m_pOST->OSMR3, osmr3);

	// Clear Dummy Status
	DEBUGMSG(TRUE, (_T("\tClearing OSSR_M3\r\n")));
	WRITE_REGISTER_ULONG(&m_pOST->OSSR, OSSR_M3);

	// Enable OSTimer 3 interrupt
	DEBUGMSG(TRUE, (_T("\tEnabling OIER\r\n")));
	oier = READ_REGISTER_ULONG(&m_pOST->OIER);
    oier |= OIER_E3;
    WRITE_REGISTER_ULONG(&m_pOST->OIER, oier);

	// Enable Watchdog Timer
	//-----------------------
	DEBUGMSG(TRUE, (_T("\tEnabling the watchdog\r\n")));
	WRITE_REGISTER_ULONG(&m_pOST->OWER, OWER_WME);	

	DEBUGMSG(TRUE, (_T("-WatchDog::WatchDog()\r\n")));
}

WatchDog::~WatchDog()
{
	DEBUGMSG(TRUE, (_T("+WatchDog::~WatchDog()\r\n")));

	if(m_pOST)
		VirtualFree(m_pOST, 0, MEM_RELEASE);

	DEBUGMSG(TRUE, (_T("-WatchDog::~WatchDog()\r\n")));
}

void WatchDog::Pet()
{
	DEBUGMSG(TRUE, (_T("+WatchDog::Pet()\r\n")));
	LONG oscr;	
	LONG osmr3;

	// Read Current OS Timer Counter
	oscr = (LONG)READ_REGISTER_ULONG(&m_pOST->OSCR);

	// calcultate the next interval
	osmr3 = oscr + m_interval;

	// Update OSTimer Match	Register
	DEBUGMSG(TRUE, (_T("\tSetting OSMR3 = 0x%x\r\n"), osmr3));
	WRITE_REGISTER_ULONG(&m_pOST->OSMR3, osmr3);

	DEBUGMSG(TRUE, (_T("-WatchDog::Pet()\r\n")));
}



// This function maps Physical address to Virtual Address
//-------------------------------------------------------

PVOID MapPhysicalAddr(PVOID dwPhysicalAddress, DWORD dwSize)
{
	LPVOID pVirtualAddr;
	PVOID PageAlinedPhysicalAddress;
	ULONG Offset;
	DWORD PageAlignedSize;
	BOOL bCopyReturn;

	// Get Page Aligned Address
	PageAlinedPhysicalAddress = PAGE_ALIGN (dwPhysicalAddress);
	Offset = (ULONG)dwPhysicalAddress - (ULONG)PageAlinedPhysicalAddress;

	// Get page Aligned Size
	PageAlignedSize = ROUND_TO_PAGES(dwSize + Offset);

	// Create reserved block of virtual memory
	pVirtualAddr = VirtualAlloc(0, PageAlignedSize, MEM_RESERVE, PAGE_NOACCESS);

    if(pVirtualAddr == NULL) 
	{
		RETAILMSG(1,(L"	MapPhysicalAddr : Error Return. \r\n"));
        return NULL;
    }

	// Map physical memory to virtual memory
 	// then /256 and use the PAGE_PHYSICAL flag
	//----------------------------------------------
    bCopyReturn = VirtualCopy (	pVirtualAddr, 
								(LPVOID)((ULONG)PageAlinedPhysicalAddress >> 8),
								PageAlignedSize, 
								PAGE_READWRITE | PAGE_NOCACHE | PAGE_PHYSICAL);

    if(bCopyReturn == FALSE) 
	{
		RETAILMSG(1,(L"	MapPhysicalAddr : return false from VirtualCopy(). \r\n"));      
        VirtualFree( pVirtualAddr, 0, MEM_RELEASE  );
        return NULL;
    }

	pVirtualAddr = (PVOID)((PUCHAR)pVirtualAddr + Offset);

	return pVirtualAddr;

}
